==> wurld-0.1.tar.gz: wurld-0.1/src/PythonObject.cc <==
/*===========================================================================*\

   $Id$

   Implementation of the python game object class.

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:

   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#include "PythonObject.h"

#include "Moveable.h"
#include <Python.h>

extern "C" PyObject *zzspcl_GameEngine_newFromPointer(GameEngine *tmpInst);
extern "C" 
   PyObject *zzspcl_PythonObject_newFromPointer(PythonObjectPtr tmpInst);

PythonObject::PythonObject() : 
   engineFrameCallback(0), 
   renderFrameCallback(0) {
}

PythonObject::~PythonObject() {
   Py_XDECREF(engineFrameCallback);
   Py_XDECREF(renderFrameCallback);
}

void PythonObject::processEngineFrame(GameEngine &engine) {
   if (engineFrameCallback) {

      PyObject *gameEnginePyObj = zzspcl_GameEngine_newFromPointer(&engine);
      PyObject *self = zzspcl_PythonObject_newFromPointer(this);
      PyObject *args = Py_BuildValue("(OO)", gameEnginePyObj, self);
      PyObject *result = PyObject_CallObject(engineFrameCallback, args);
      if (!result)
	 PyErr_Print();
      else
	 Py_DECREF(result);
   }

   if (moveable)
      moveable->processEngineFrame(engine);
}

void PythonObject::processRenderFrame(GameEngine &engine, Float elapsed) {
   if (renderFrameCallback) {
      PyObject *gameEnginePyObj = zzspcl_GameEngine_newFromPointer(&engine);
      PyObject *args = Py_BuildValue("Od", gameEnginePyObj, elapsed);
      PyObject *result = PyObject_CallObject(engineFrameCallback, args);
      if (!result)
	 PyErr_Print();
      else
	 Py_DECREF(result);
   }

   if (moveable)
      moveable->processRenderFrame(engine, elapsed);
}

void PythonObject::render(Scene &scene) {
   if (moveable)
      moveable->render(scene);
}

void PythonObject::setEngineFrameCallback(PyObject *callback) {
   // same object, do nothing
   if (callback == engineFrameCallback)
      return;

   // if there's an existing object, unlink it
   Py_XDECREF(engineFrameCallback);

   Py_INCREF(callback);
   engineFrameCallback = callback;
}

void PythonObject::setRenderFrameCallback(PyObject *callback) {
   // same object, do nothing
   if (callback == renderFrameCallback)
      return;

   // if there's an existing object, unlink it
   Py_XDECREF(renderFrameCallback);

   Py_INCREF(callback);
   renderFrameCallback = callback;
}

==> wurld-0.3.tar.gz: wurld-0.3/src/PythonObject.cc <==
/*===========================================================================*\

   $Id$

   Implementation of the python game object class.

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:

   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#include "PythonObject.h"

#include "Moveable.h"
#include <Python.h>

extern "C" PyObject *zzspcl_GameEngine_newFromPointer(GameEngine *tmpInst);
extern "C" 
   PyObject *zzspcl_PythonObject_newFromPointer(PythonObjectPtr tmpInst);

PythonObject::PythonObject() : 
   engineFrameCallback(0), 
   renderFrameCallback(0),
   collisionCallback(0) {
}

PythonObject::~PythonObject() {
   if (moveable)
      moveable->setCollisionHandler(0);
   Py_XDECREF(engineFrameCallback);
   Py_XDECREF(renderFrameCallback);
}

void PythonObject::processEngineFrame(GameEngine &engine) {
   if (engineFrameCallback) {

      PyObject *gameEnginePyObj = zzspcl_GameEngine_newFromPointer(&engine);
      PyObject *self = zzspcl_PythonObject_newFromPointer(this);
      PyObject *args = Py_BuildValue("(OO)", gameEnginePyObj, self);
      PyObject *result = PyObject_CallObject(engineFrameCallback, args);
      if (!result)
	 PyErr_Print();
      else
	 Py_DECREF(result);
   }

   if (moveable)
      moveable->processEngineFrame(engine);
}

void PythonObject::processRenderFrame(GameEngine &engine, Float elapsed) {
   if (renderFrameCallback) {
      PyObject *gameEnginePyObj = zzspcl_GameEngine_newFromPointer(&engine);
      PyObject *args = Py_BuildValue("(Od)", gameEnginePyObj, elapsed);
      PyObject *result = PyObject_CallObject(renderFrameCallback, args);
      if (!result)
	 PyErr_Print();
      else
	 Py_DECREF(result);
   }

   if (moveable)
      moveable->processRenderFrame(engine, elapsed);
}

void PythonObject::render(Scene &scene) {
   if (moveable)
      moveable->render(scene);
}

void PythonObject::handleCollision(const Collidable &collidable) {
   if (collisionCallback) {
      PyObject *args = PyTuple_New(0);
      PyObject *result = PyObject_CallObject(collisionCallback, args);
      if (!result)
         PyErr_Print();
      else
         Py_DECREF(result);
   }
}

void PythonObject::setCallback(PyObject *&callbackVar, PyObject *callback) {
   // same object, do nothing
   if (callback == callbackVar)
      return;

   // translate None to null
   if (callback == Py_None)
      callback = 0;

   // if there's an existing object, unlink it
   Py_XDECREF(callbackVar);

   Py_XINCREF(callback);
   callbackVar = callback;
}


void PythonObject::setEngineFrameCallback(PyObject *callback) {
   setCallback(engineFrameCallback, callback);
}

void PythonObject::setRenderFrameCallback(PyObject *callback) {
   setCallback(renderFrameCallback, callback);
}

void PythonObject::setCollisionCallback(PyObject *callback) {
   setCallback(collisionCallback, callback);
}

void PythonObject::setMoveable(const MoveablePtr &newMoveable) {
   if (moveable)
      moveable->setCollisionHandler(0);

   moveable = newMoveable;
   moveable->setCollisionHandler(this);
}
